# -*- coding: UTF-8 -*-
''' Upgrade the database tables if needed'''
from __future__ import print_function
from datetime import datetime
from euphorie.client import model
from logging import getLogger
from pkg_resources import get_distribution
from pkg_resources import parse_version
from Products.Five import zcml
from sqlalchemy import sql
from sqlalchemy.engine.reflection import Inspector
from sys import argv
from transaction import commit
from z3c.saconfig import Session


logger = getLogger(__name__)
euphorie_version = get_distribution('euphorie').parsed_version

try:
    config = argv[1]
except IndexError:
    config = 'parts/instance/etc/package-includes/999-additional-overrides.zcml'  # noqa: E501

zcml.load_config(config)
session = Session()
inspector = Inspector.from_engine(session.bind)


def execute(statement):
    ''' Execute the given SQL statement and commit immediately after it is
    executed
    '''
    logger.info(statement)
    session.execute(statement)
    session.execute('COMMIT;')


def create_missing_tables():
    ''' This will create the missing tables
    '''
    model.metadata.create_all(Session.bind, checkfirst=True)


def add_group_id_to_account():
    ''' A new 'group_id' column has been added to the 'account' table
    '''
    for column in inspector.get_columns('account'):
        if 'group_id' == column['name']:
            return
    statement = (
        '''
        ALTER TABLE account
            ADD COLUMN group_id character varying(32);
        ALTER TABLE account
            ADD CONSTRAINT account_group_id_fkey
            FOREIGN KEY (group_id)
            REFERENCES "group" (group_id) MATCH SIMPLE
            ON UPDATE NO ACTION
            ON DELETE NO ACTION;
        '''
    )
    execute(statement)


def add_brand_to_session():
    ''' A new 'brand' column has been added to the 'session' table
    '''
    for column in inspector.get_columns('session'):
        if 'brand' == column['name']:
            return

    statement = (
        '''
        ALTER TABLE session
            ADD COLUMN brand character varying(64);
        '''
    )
    execute(statement)


def add_brand_to_group():
    ''' A new 'brand' column has been added to the 'group' table
    '''
    for column in inspector.get_columns('group'):
        if 'brand' == column['name']:
            return

    statement = (
        '''
        ALTER TABLE "group"
            ADD COLUMN brand character varying(64);
        '''
    )
    execute(statement)


def add_group_id_to_session():
    ''' A new 'group_id' column has been added to the 'session' table
    '''
    for column in inspector.get_columns('session'):
        if 'group_id' == column['name']:
            return

    statement = (
        '''
        ALTER TABLE session
            ADD COLUMN group_id character varying(32);
        ALTER TABLE session
            ADD CONSTRAINT session_group_id_fkey
            FOREIGN KEY (group_id)
            REFERENCES public."group" (group_id) MATCH SIMPLE
            ON UPDATE NO ACTION
            ON DELETE NO ACTION;
        '''
    )
    execute(statement)


def add_published_to_session():
    ''' A new 'published' column has been added to the 'session' table
    '''
    for column in inspector.get_columns('session'):
        if 'published' == column['name']:
            return

    statement = (
        '''
        ALTER TABLE session
            ADD COLUMN published timestamp with time zone;
        '''
    )
    execute(statement)


def add_last_modifier_id_to_session():
    ''' A new 'last_modifier_id' column has been added to the 'session' table
    '''
    for column in inspector.get_columns('session'):
        if 'last_modifier_id' == column['name']:
            return

    statement = (
        '''
        ALTER TABLE session
            ADD COLUMN last_modifier_id integer;
        ALTER TABLE session
            ADD CONSTRAINT session_last_modifier_id_fkey
            FOREIGN KEY (last_modifier_id)
            REFERENCES public.account (id) MATCH SIMPLE
            ON UPDATE NO ACTION
            ON DELETE NO ACTION;
        '''
    )
    execute(statement)


def add_last_publisher_id_to_session():
    ''' A new 'last_publisher_id' column has been added to the 'session' table
    '''
    for column in inspector.get_columns('session'):
        if 'last_publisher_id' == column['name']:
            return

    statement = (
        '''
        ALTER TABLE session
            ADD COLUMN last_publisher_id integer;
        ALTER TABLE session
            ADD CONSTRAINT session_last_publisher_id_fkey
            FOREIGN KEY (last_publisher_id)
            REFERENCES public.account (id) MATCH SIMPLE
            ON UPDATE NO ACTION
            ON DELETE NO ACTION;
        '''
    )
    execute(statement)


def add_custom_description_to_risk():
    ''' A new 'custom_description' column has been added to the 'risk' table
    '''
    print("add custom description to risk")
    for column in inspector.get_columns('risk'):
        if 'custom_description' == column['name']:
            return

    statement = (
        '''
        ALTER TABLE risk
            ADD COLUMN custom_description text;
        '''
    )
    execute(statement)


def hash_passwords():
    ''' We want the passwords stored in the account table to be encrypted
    '''
    accounts = session.query(model.Account).filter(
        sql.or_(
            sql.not_(model.Account.account_type == 'guest'),
            model.Account.account_type == None  # noqa: E711
        )
    )
    total = float(accounts.count())
    start = datetime.now()
    print(
        '{} - {} accounts to convert'.format(
            start.strftime('%Y/%m/%d %H:%M:%S'), int(total)
        )
    )
    cnt = 0
    for account in accounts:
        account.hash_password()
        cnt += 1
        if cnt % 500 == 0:
            print(
                '{} - {} accounts converted ({:2.2f}%)'.format(
                    datetime.now().strftime('%Y/%m/%d %H:%M:%S'),
                    cnt,
                    cnt / total * 100,
                )
            )
            print('    {}'.format(account.loginname))
    print(
        '{} accounts processed. Finished after {}'.format(
            cnt,
            datetime.now() - start,
        )
    )
    commit()


def main():
    # It is always a good idea to run this one
    create_missing_tables()
    if euphorie_version < parse_version('10.0.1'):
        add_group_id_to_account()
        add_brand_to_session()
        add_group_id_to_session()
        add_published_to_session()
        add_last_modifier_id_to_session()
        add_last_publisher_id_to_session()
        hash_passwords()
    if euphorie_version < parse_version('10.0.4'):
        add_brand_to_group()
    if euphorie_version < parse_version('11.0.5'):
        add_custom_description_to_risk()

if __name__ == "__main__":
    main()



# coding=utf-8
from euphorie.client.api.authentication import generate_token
from euphorie.client.api.risk import ActionPlan
from euphorie.client.api.risk import Identification
from euphorie.client.api.risk import View
from euphorie.client.api.tests.utils import _setup_session
from euphorie.client.model import Risk
from euphorie.client.model import Session
from euphorie.content.solution import Solution
from euphorie.testing import EuphorieFunctionalTestCase
from euphorie.testing import EuphorieIntegrationTestCase
from plone.namedfile.file import NamedBlobImage
from zope.publisher.browser import TestRequest

import json


DUMMY_GIF = 'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\xff' \
            '\xff\xff!\xf9\x04\x01\x00\x00\x01\x00,\x00\x00\x00' \
            '\x00\x01\x00\x01\x00\x00\x02\x01L\x00;'


class ViewTests(EuphorieIntegrationTestCase):

    def test_do_GET_minimal(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        survey['1'].title = u'Module title.'
        risk = survey['1']['2']
        risk.title = u'Everything is under control.'
        risk.problem_description = u'Not everything under control.'
        risk.description = None
        risk.evaluation_method = 'direct'
        risk.image = None
        request = TestRequest()
        request.survey = survey
        risk = Session.query(Risk).first()
        view = View(risk, request)
        response = view.do_GET()
        self.assertEqual(
            set(response),
            set([
                'id', 'type', 'title', 'module-title', 'problem-description',
                'show-not-applicable', 'evaluation-method', 'present',
                'priority', 'comment'
            ])
        )
        self.assertEqual(response['id'], 2)
        self.assertEqual(response['type'], 'risk')
        self.assertEqual(response['title'], u'Everything is under control.')
        self.assertEqual(response['module-title'], u'Module title.')
        self.assertEqual(
            response['problem-description'], u'Not everything under control.'
        )
        self.assertEqual(response['show-not-applicable'], False)
        self.assertEqual(response['evaluation-method'], 'direct')
        self.assertEqual(response['present'], None)
        self.assertEqual(response['priority'], u'high')

    def test_do_GET_full(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        risk = survey['1']['2']
        risk.description = u'<p>Simple description</p>'
        risk.legal_reference = u'<p>Catch 22</p>'
        risk.evaluation_method = 'calculated'
        risk.image2 = NamedBlobImage(
            data=DUMMY_GIF, contentType='image/gif', filename=u'dummy.gif'
        )
        risk.caption2 = u'Secondary Image'
        risk['3'] = Solution(
            description=u'Standard solution 1',
            action_plan=u'Dummy plan',
            requirements=u'Dummy requirements'
        )
        request = TestRequest()
        request.survey = survey
        risk = Session.query(Risk).first()
        risk.skip_children = True
        view = View(risk, request)
        response = view.do_GET()
        self.assertEqual(
            set(response),
            set([
                'id', 'type', 'title', 'module-title', 'problem-description',
                'show-not-applicable', 'evaluation-method', 'present',
                'priority', 'comment', 'description', 'legal-reference',
                'evaluation-algorithm', 'frequency', 'frequency-options',
                'effect', 'effect-options', 'probability',
                'probability-options', 'images', 'standard-solutions'
            ])
        )
        self.assertEqual(response['description'], u'<p>Simple description</p>')
        self.assertEqual(response['legal-reference'], u'<p>Catch 22</p>')
        self.assertEqual(len(response['images']), 2)
        self.assertEqual(len(response['standard-solutions']), 1)
        self.assertEqual(
            response['standard-solutions'][0], {
                'description': u'Standard solution 1',
                'action-plan': u'Dummy plan',
                'prevention-plan': None,
                'requirements': u'Dummy requirements'
            }
        )

    def test_do_GET_use_vocabulary_token(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        risk = survey['1']['2']
        risk.title = u'Everything is under control.'
        risk.problem_description = u'Not everything under control.'
        risk.description = None
        risk.evaluation_method = 'calculated'
        request = TestRequest()
        request.survey = survey
        risk = Session.query(Risk).first()
        risk.probability = 3
        risk.frequency = 7
        view = View(risk, request)
        response = view.do_GET()
        self.assertEqual(response['probability'], 'medium')
        self.assertEqual(response['frequency'], 'constant')
        self.assertEqual(response['effect'], None)


class IdentificationTests(EuphorieIntegrationTestCase):

    def Identification(self, *a, **kw):
        return Identification(*a, **kw)

    def test_do_PUT_missing_present_value(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        risk = Session.query(Risk).first()
        view = self.Identification(risk, request)
        view.input = {}
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')

    def test_do_PUT_present_invalid_value(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        risk = Session.query(Risk).first()
        view = self.Identification(risk, request)
        view.input = {'present': 'foo'}
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')

    def test_do_PUT_keep_existing_comment(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        risk = Session.query(Risk).first()
        risk.comment = u'Original comment'
        view = self.Identification(risk, request)
        view.input = {'present': 'yes'}
        response = view.do_PUT()
        self.assertEqual(response['comment'], u'Original comment')
        self.assertEqual(risk.comment, u'Original comment')


class ActionPlanTests(EuphorieIntegrationTestCase):

    def ActionPlan(self, *a, **kw):
        return ActionPlan(*a, **kw)

    def test_do_PUT_set_priority_for_top5_risk_not_allowed(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        risk = survey['1']['2']
        risk.type = 'top5'
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        risk = Session.query(Risk).first()
        view = self.ActionPlan(risk, request)
        view.input = {'priority': 'low'}
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')

    def test_do_PUT_set_priority_for_normal_risk(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)

        risk = survey['1']['2']
        risk.type = 'risk'
        request = TestRequest()
        request.survey = survey
        request.survey_session = survey_session
        risk = Session.query(Risk).first()
        view = self.ActionPlan(risk, request)
        view.input = {'priority': 'low'}
        response = view.do_PUT()
        self.assertEqual(response['priority'], 'low')
        self.assertEqual(risk.priority, 'low')


class BrowserTests(EuphorieFunctionalTestCase):

    def test_get(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        browser = self.get_browser()
        browser.addHeader('X-Euphorie-Token', generate_token(account))
        browser.open('http://nohost/plone/client/api/users/1/sessions/1/1/1')
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(response['type'], 'risk')

    def test_get_translation(self):
        self.loginAsPortalOwner()
        (account, survey, survey_session) = _setup_session(self.portal)
        survey.language = 'nl'
        browser = self.get_browser()
        browser.addHeader('X-Euphorie-Token', generate_token(account))
        browser.open('http://nohost/plone/client/api/users/1/sessions/1/1/1')
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        options = dict((opt['value'], opt['title'])
                       for opt in response['frequency-options'])
        self.assertEqual(options['constant'], u'Voortdurend')



# coding=utf-8
from euphorie.testing import EuphorieFunctionalTestCase

import json
import mock


class GenericErrorTests(EuphorieFunctionalTestCase):

    def test_View(self):
        with mock.patch(
            'euphorie.client.api.entry.View.do_GET', sideEffect=RuntimeError
        ):
            browser = self.get_browser()
            browser.raiseHttpErrors = False
            browser.open('http://nohost/plone/client/api')
            self.assertTrue(browser.headers['Status'].startswith('500'))
            response = json.loads(browser.contents)
            self.assertEqual(set(response), set(['type', 'message']))
            self.assertEqual(response['type'], 'error')
            self.assertEqual(
                response['message'], u'An unknown error occurred.'
            )


class NotFoundViewTests(EuphorieFunctionalTestCase):

    def test_404_handled(self):
        browser = self.get_browser()
        browser.raiseHttpErrors = False
        browser.open('http://nohost/plone/client/api/unknown')
        self.assertTrue(browser.headers['Status'].startswith('404'))
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(set(response), set(['type', 'message']))
        self.assertEqual(response['type'], 'error')
        self.assertEqual(response['message'], 'Unknown resource requested.')



# coding=utf-8
from euphorie.testing import EUPHORIE_FUNCTIONAL_TESTING
from plone.testing import layered

import doctest
import glob
import os.path
import unittest


def test_suite():
    location = os.path.dirname(__file__) or '.'
    doctests = [
        'stories/' + os.path.basename(test)
        for test in glob.glob(os.path.join(location, 'stories', '*.txt'))
    ]

    options = (
        doctest.REPORT_ONLY_FIRST_FAILURE | doctest.ELLIPSIS
        | doctest.NORMALIZE_WHITESPACE
    )

    suites = [
        layered(
            doctest.DocFileSuite(
                test,
                optionflags=options,
                module_relative=True,
                package='euphorie.content.tests',
                encoding='utf-8',
            ),
            layer=EUPHORIE_FUNCTIONAL_TESTING,
        ) for test in doctests
    ]
    return unittest.TestSuite(suites)



# coding=utf-8
from euphorie.client.api.sector import View
from euphorie.client.sector import ClientSector
from euphorie.client.tests.utils import addSurvey
from euphorie.content.survey import Survey
from euphorie.content.tests.utils import BASIC_SURVEY
from euphorie.ghost import PathGhost
from euphorie.testing import EuphorieFunctionalTestCase
from euphorie.testing import EuphorieIntegrationTestCase
from plone.folder.default import DefaultOrdering
from zope.annotation.attribute import AttributeAnnotations
from zope.component import provideAdapter
from zope.publisher.browser import TestRequest

import json


class ViewTests(EuphorieIntegrationTestCase):

    def setUp(self):
        provideAdapter(AttributeAnnotations)
        provideAdapter(DefaultOrdering)

    def View(self, *a, **kw):
        return View(*a, **kw)

    def test_plain_view(self):
        sector = ClientSector(id='ict', title=u'ICT')
        response = self.View(sector, TestRequest()).do_GET()
        self.assertTrue(isinstance(response, dict))
        self.assertEqual(set(response), set(['id', 'title', 'surveys']))
        self.assertEqual(response['id'], 'ict')
        self.assertEqual(response['title'], u'ICT')
        self.assertEqual(response['surveys'], [])

    def test_survey_info(self):
        sector = ClientSector(id='ict', title=u'ICT')
        sector._setOb(
            'gaming',
            Survey(id='gaming', title=u'Gaming', language='nl'),
        )
        response = self.View(sector, TestRequest()).do_GET()
        self.assertEqual(len(response['surveys']), 1)
        survey_info = response['surveys'][0]
        self.assertTrue(isinstance(survey_info, dict))
        self.assertEqual(set(survey_info), set(['id', 'title', 'language']))
        self.assertEqual(survey_info['id'], 'gaming')
        self.assertEqual(survey_info['title'], u'Gaming')
        self.assertEqual(survey_info['language'], 'nl')

    def test_ignore_non_survey_child(self):
        sector = ClientSector(id='ict', title=u'ICT')
        sector['gaming'] = PathGhost('gaming')
        response = self.View(sector, TestRequest()).do_GET()
        self.assertEqual(response['surveys'], [])

    def test_survey_info_filter_by_language(self):
        sector = ClientSector(id='ict', title=u'ICT')
        sector._setOb(
            'gaming',
            Survey(id='gaming', title=u'Gaming', language='nl'),
        )
        request = TestRequest()
        request.form['language'] = 'en'
        response = self.View(sector, request).do_GET()
        self.assertEqual(len(response['surveys']), 0)
        request.form['language'] = 'nl'
        response = self.View(sector, request).do_GET()
        self.assertEqual(len(response['surveys']), 1)


class BrowserTests(EuphorieFunctionalTestCase):

    def test_get(self):
        self.loginAsPortalOwner()
        addSurvey(self.portal, BASIC_SURVEY)
        browser = self.get_browser()
        browser.open('http://nohost/plone/client/api/surveys/nl/ict')
        response = json.loads(browser.contents)
        self.assertEqual(response['id'], 'ict')



# coding=utf-8
from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from euphorie.content.api.entry import access_api
from euphorie.content.countrymanager import ICountryManager
from plone import api
from plone.memoize.view import memoize_contextless
from plone.protect.interfaces import IDisableCSRFProtection
from plonetheme.nuplone.skin.interfaces import NuPloneSkin
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.Five import BrowserView
from Products.membrane.interfaces.user import IMembraneUser
from time import time
from zope.component import adapter
from zope.interface import alsoProvides
from ZPublisher.BaseRequest import DefaultPublishTraverse

import six


class Frontpage(BrowserView):

    def __call__(self):
        user = getSecurityManager().getUser()
        if IMembraneUser.providedBy(user):
            mt = getToolByName(self.context, "membrane_tool")
            obj = mt.getUserObject(user_id=user.getUserId())
            if ICountryManager.providedBy(obj):
                return self.request.response.redirect(
                    aq_parent(obj).absolute_url()
                )
            return self.request.response.redirect(obj.absolute_url())
        portal = aq_inner(self.context)
        return self.request.response.redirect(
            "%s/sectors/" % portal.absolute_url()
        )


@adapter(IPloneSiteRoot, NuPloneSkin)
class SitePublishTraverser(DefaultPublishTraverse):
    """Publish traverser to manage access to the CMS API.

    This traverser marks the request with IClientSkinLayer. We can not use
    BeforeTraverseEvent since in Zope 2 that is only fired for site objects.
    """

    def publishTraverse(self, request, name):
        if name == 'api':
            return access_api(request).__of__(self.context)
        return super(SitePublishTraverser, self).publishTraverse(request, name)


class EuphorieRefreshResourcesTimestamp(BrowserView):

    @property
    @memoize_contextless
    def resources_timestamp(self):
        return api.portal.get_registry_record(
            'euphorie.deployment.resources_timestamp',
            default='',
        )

    def refresh_timestamp(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        api.portal.set_registry_record(
            'euphorie.deployment.resources_timestamp',
            six.text_type(int(time())),
        )

    def __call__(self):
        ''' Refresh the registry record that adds a timestamp
        to the resources urls
        '''
        self.refresh_timestamp()
        return 'OK'



from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer
from plonetheme.nuplone.z3cform.interfaces import INuPloneFormLayer


class IClientSkinLayer(IDefaultBrowserLayer, INuPloneFormLayer):
    """Zope skin layer for the online client.
    """


class IIdentificationPhaseSkinLayer(Interface):
    """Skin layer used during the identification phase."""


class ICustomizationPhaseSkinLayer(Interface):
    """Skin layer used during the custom risk phase."""


class IEvaluationPhaseSkinLayer(Interface):
    """Skin layer used during the evaluation phase."""


class IActionPlanPhaseSkinLayer(Interface):
    """Skin layer used during the action plan phase."""


class IReportPhaseSkinLayer(Interface):
    """Skin layer used during the report phase."""


# Special case for individual countries...

class IItaly(Interface):
    """Skin layer to mark the country Italy"""

class IItalyIdentificationPhaseSkinLayer(IItaly, IIdentificationPhaseSkinLayer):
    """Skin layer to mark the country Italy"""

class IItalyCustomizationPhaseSkinLayer(IItaly, ICustomizationPhaseSkinLayer):
    """Skin layer to mark the country Italy"""

class IItalyEvaluationPhaseSkinLayer(IItaly, IEvaluationPhaseSkinLayer):
    """Skin layer to mark the country Italy"""

class IItalyActionPlanPhaseSkinLayer(IItaly, IActionPlanPhaseSkinLayer):
    """Skin layer to mark the country Italy"""

class IItalyReportPhaseSkinLayer(IItaly, IReportPhaseSkinLayer):
    """Skin layer to mark the country Italy"""


class IFrance(Interface):
    """Skin layer to mark the country France"""

class IFranceIdentificationPhaseSkinLayer(IFrance, IIdentificationPhaseSkinLayer):
    """Skin layer to mark the country France"""

class IFranceCustomizationPhaseSkinLayer(IFrance, ICustomizationPhaseSkinLayer):
    """Skin layer to mark the country France"""

class IFranceEvaluationPhaseSkinLayer(IFrance, IEvaluationPhaseSkinLayer):
    """Skin layer to mark the country France"""

class IFranceActionPlanPhaseSkinLayer(IFrance, IActionPlanPhaseSkinLayer):
    """Skin layer to mark the country France"""

class IFranceReportPhaseSkinLayer(IFrance, IReportPhaseSkinLayer):
    """Skin layer to mark the country France"""



# coding=utf-8
from euphorie.client.session import SessionManager
from logging import getLogger
from Products.CMFPlone.utils import safe_unicode
from Products.Five import BrowserView
from euphorie.client.update import redirectOnSurveyUpdate

logger = getLogger(__name__)


class TrainingSlide(BrowserView):
    """ Template / macro to hold the training slide markup
    Currently not active in default Euphorie
    """

    def __call__(self):
        return self


class TrainingView(BrowserView):
    """ The view that shows the main-menu Training module
    Currently not active in default Euphorie
    """
    variation_class = "variation-risk-assessment"

    def __call__(self):
        if redirectOnSurveyUpdate(self.request):
            return
        pptx_view = self.context.restrictedTraverse('pptx', None)
        if pptx_view:
            self.slide_data = pptx_view.get_data()
        else:
            self.slide_data = None
        if self.request.environ["REQUEST_METHOD"] == "POST":

            for entry in self.request.form:
                if entry.startswith("training_notes"):
                    try:
                        index = int(entry.split("-")[-1])
                    except:
                        continue
                    risk_data = self.slide_data['slides'][index]
                    sql_item = risk_data['row']
                    value = safe_unicode(self.request[entry])
                    sql_item.training_notes = value
                    self.slide_data['slides'][index]['row'] = sql_item
                    risk_data['training_notes'] = value
                SessionManager.session.touch()

        return self.index()



# coding=utf-8
from euphorie.client.api.users import Authenticate
from euphorie.client.api.users import View
from euphorie.client.model import Account
from euphorie.testing import EuphorieFunctionalTestCase
from euphorie.testing import EuphorieIntegrationTestCase
from z3c.saconfig import Session

import json
import mock
import unittest


class ViewTests(EuphorieIntegrationTestCase):

    def test_do_POST_valid_data(self):
        view = View(None, self.request.clone())
        view.input = {'login': 'john', 'password': 'jane'}
        with mock.patch(
            'euphorie.client.api.users.generate_token'
        ) as mock_generate_token:
            mock_generate_token.return_value = 'token'
            response = view.do_POST()
        self.assertEqual(response['type'], 'user')


class AuthenticateTests(unittest.TestCase):

    def test_render_no_data_provided(self):
        view = Authenticate(None, mock.Mock())
        view.input = None
        response = view.do_POST()
        self.assertEqual(response['type'], 'error')

    def test_render_missing_data(self):
        view = Authenticate(None, mock.Mock())
        view.input = {'login': 'foo'}
        response = view.do_POST()
        self.assertEqual(response['type'], 'error')

    def test_render_bad_login(self):
        request = mock.Mock()
        view = Authenticate(None, request)
        view.input = {'login': 'foo', 'password': 'bar'}
        with mock.patch('euphorie.client.api.users.authenticate') as mock_auth:
            mock_auth.return_value = None
            response = view.do_POST()
            mock_auth.assert_called_once_with('foo', 'bar')
            self.assertEqual(response['type'], 'error')
            self.assertEqual(response['message'], 'Invalid credentials')
            request.response.setStatus.assert_called_once_with(403)


class AuthenticateIntegrationTests(EuphorieIntegrationTestCase):

    def test_render_valid_login(self):
        view = Authenticate(None, self.request.clone())
        view.input = {'login': 'foo', 'password': 'bar'}
        with mock.patch('euphorie.client.api.users.authenticate') as mock_auth:
            with mock.patch(
                'euphorie.client.api.users.generate_token'
            ) as mock_generate_token:
                with mock.patch('euphorie.client.api.users.user_info') as info:
                    view.sessions = mock.Mock(return_value='sessions')
                    info.return_value = {'foo': 'bar'}
                    mock_auth.return_value = account = mock.Mock()
                    mock_generate_token.return_value = 'auth-token'
                    response = view.do_POST()
                    mock_auth.assert_called_once_with('foo', 'bar')
                    info.assert_called_once_with(account, view.request)
                    self.assertEqual(response['foo'], 'bar')
                    self.assertEqual(response['token'], 'auth-token')


class AuthenticateBrowserTests(EuphorieFunctionalTestCase):

    def test_authentication_failure(self):
        browser = self.get_browser()
        browser.raiseHttpErrors = False
        browser.post(
            'http://nohost/plone/client/api/users/authenticate',
            '{"login": "jane", "password": "john"}'
        )
        self.assertTrue(browser.headers['Status'].startswith('403'))
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(response['type'], 'error')

    def test_authentication_success(self):
        Session.add(Account(loginname='john', password=u'jane'))
        browser = self.get_browser()
        browser.handleErrors = False
        browser.post(
            'http://nohost/plone/client/api/users/authenticate',
            '{"login": "john", "password": "jane"}'
        )
        response = json.loads(browser.contents)
        self.assertEqual(response['type'], 'user')



# coding=utf-8
from euphorie.client.profile import set_session_profile
from euphorie.client.session import create_survey_session
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.content.tests.utils import BASIC_SURVEY


def _setup_session(portal):
    addSurvey(portal, BASIC_SURVEY)
    survey = portal.client['nl']['ict']['software-development']
    account = addAccount(password='secret')
    survey_session = create_survey_session(u'Dummy session', survey, account)
    survey_session = set_session_profile(survey, survey_session, {})
    return (account, survey, survey_session)



# coding=utf-8
from ..survey import View
from euphorie.content.module import IModule
from euphorie.content.profilequestion import IProfileQuestion
from euphorie.content.survey import handleSurveyUnpublish
from euphorie.content.survey import Survey
from euphorie.testing import EuphorieIntegrationTestCase
from plone.app.layout.globals.context import ContextState
from plone.folder.default import DefaultOrdering
from zope.annotation.attribute import AttributeAnnotations
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.component import provideAdapter
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.publisher.browser import TestRequest

import Acquisition
import mock
import unittest


class Mock(Acquisition.Explicit):

    def __init__(self, **kwargs):
        for (key, value) in kwargs.items():
            setattr(self, key, value)

    def absolute_url(self):
        return "http://nohost/%s" % self.id

    def manage_fixupOwnershipAfterAdd(self):
        pass


class ViewTests(EuphorieIntegrationTestCase):

    def setUp(self):
        super(ViewTests, self).setUp()
        provideAdapter(AttributeAnnotations)
        provideAdapter(DefaultOrdering)
        provideAdapter(
            ContextState,
            adapts=(Interface, Interface),
            provides=Interface,
            name="plone_context_state"
        )
        # grok makes unit testing extremely painful
        View.__view_name__ = "View"
        View.module_info = Mock()
        View.module_info.package_dotted_name = 'euphorie.content.survey.View'

    def tearDown(self):
        super(ViewTests, self).tearDown()
        del View.__view_name__
        del View.module_info

    def _request(self):
        req = TestRequest()
        alsoProvides(req, IAttributeAnnotatable)
        return req

    def test_update_no_children(self):
        survey = Survey()
        view = View(survey, self._request())
        view.update()
        self.assertEqual(view.children, [])

    def test_update_with_profile(self):
        survey = Survey()
        child = Mock(id="child", title=u"Child")
        alsoProvides(child, IProfileQuestion)
        survey['child'] = child
        view = View(survey, self._request())
        view._morph = mock.Mock(return_value='info')
        view.update()
        self.assertEqual(view.children, ['info'])

    def test_update_with_module(self):
        survey = Survey()
        child = Mock(id="child", title=u"Child")
        alsoProvides(child, IModule)
        survey['child'] = child
        view = View(survey, self._request())
        view._morph = mock.Mock(return_value='info')
        view.update()
        self.assertEqual(view.children, ['info'])

    def test_update_other_child(self):
        survey = Survey()
        view = View(survey, self._request())
        child = Mock(id='child', title=u'Child')
        survey['child'] = child
        view.update()
        self.assertEqual(view.children, [])

    def test_moprh(self):
        child = Mock(id='child', title=u'Child')
        view = View(None, self._request())
        self.assertEqual(
            view._morph(child), {
                'id': 'child',
                'title': u'Child',
                'url': 'http://nohost/child'
            }
        )


class HandleSurveyUnpublishTests(unittest.TestCase):

    def handleSurveyUnpublish(self, *a, **kw):
        return handleSurveyUnpublish(*a, **kw)

    def testRemovePublishedFromSurvey(self):
        surveygroup = Mock(published=None)
        surveygroup.survey = Mock(id='survey', published='yes')
        self.handleSurveyUnpublish(surveygroup.survey, None)
        self.assertTrue(not hasattr(surveygroup.survey, 'published'))

    def testUpdatateSurveygroupIfCurrentlyPublished(self):
        surveygroup = Mock(published='survey')
        surveygroup.survey = Mock(id='survey', published='yes')
        self.handleSurveyUnpublish(surveygroup.survey, None)
        self.assertEqual(surveygroup.published, None)

    def testUpdatateSurveygroupIfOtherPublished(self):
        surveygroup = Mock(published='other')
        surveygroup.survey = Mock(id='survey', published='yes')
        self.handleSurveyUnpublish(surveygroup.survey, None)
        self.assertEqual(surveygroup.published, 'other')



from .. import model
from ..session import SessionManagerFactory
from ..utils import setRequest
from .database import DatabaseTests
from .test_update import TreeTests
from .utils import testRequest
from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManagement import setSecurityManager
from euphorie.client import session
from plone import api
from Products.CMFCore.interfaces import ISiteRoot
from zope.testing.cleanup import cleanUp

import mock
import unittest
import zope.component


class Mock(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)


class CachingTests(unittest.TestCase):

    def testCachedSession(self):
        request = testRequest()
        marker = []
        request.other["euphorie.session"] = marker
        mgr = session.SessionManagerFactory()

        setRequest(request)
        try:
            self.failUnless(mgr.session is marker)
        finally:
            setRequest(None)

    def testCachedSessionId(self):
        request = testRequest()
        marker = []
        ses = Mock(id=marker)
        request.other["euphorie.session"] = ses
        mgr = session.SessionManagerFactory()
        setRequest(request)
        try:
            self.failUnless(mgr.id is marker)
        finally:
            setRequest(None)


class SessionCreationTests(DatabaseTests):

    def setUp(self):
        super(SessionCreationTests, self).setUp()
        zope.component.provideUtility(self, ISiteRoot)

    def tearDown(self):
        super(SessionCreationTests, self).tearDown()
        cleanUp()

    def testNewSession(self):

        request = testRequest()
        mgr = session.SessionManagerFactory()
        request.client = survey = object()
        setRequest(request)
        try:
            account = model.Account(loginname="jane", password=u"john")
            with mock.patch('euphorie.client.session.create_survey_session') \
                    as mock_create:
                survey_session = mock.Mock()
                survey_session.id = 43
                mock_create.return_value = survey_session
                ses = mgr.start(u"Test session", survey, account)
                self.assertTrue(ses is survey_session)
                self.failUnless(request.other["euphorie.session"] is ses)
        finally:
            setRequest(None)


class SessionManagerFactoryTests(TreeTests):

    def setUp(self):
        setRequest(testRequest())
        super(SessionManagerFactoryTests, self).setUp()

    def tearDown(self):
        setRequest(None)
        super(SessionManagerFactoryTests, self).tearDown()

    def SessionManagerFactory(self, *a):
        return SessionManagerFactory(*a)

    def test_session_no_session_open(self):
        with mock.patch(
            'euphorie.client.session.SessionManagerFactory.id',
            new_callable=mock.PropertyMock
        ) as mock_id:
            mock_id.return_value = None
            mgr = self.SessionManagerFactory()
            self.assertEqual(mgr.session, None)

    def test_session_valid_session_open(self):
        session = self.createSurveySession()
        with mock.patch(
            'euphorie.client.session.SessionManagerFactory.id',
            new_callable=mock.PropertyMock
        ) as mock_id:
            mock_id.return_value = session.id
            mgr = self.SessionManagerFactory()
            sm = getSecurityManager()
            try:
                newSecurityManager(None, session.account)
                self.assertTrue(mgr.session is session)
            finally:
                setSecurityManager(sm)

    def test_resume_enforce_same_account(self):
        mgr = self.SessionManagerFactory()
        victim = model.Account(loginname="test", password=u"test")
        attacker = model.Account(loginname="evil", password=u"layer")
        session = model.SurveySession(account=victim)
        with self._get_view('webhelpers', self.portal):
            with api.env.adopt_user(user=attacker):
                self.assertRaises(ValueError, mgr.resume, session)



# coding=utf-8
from euphorie.client import model
from euphorie.client import utils
from euphorie.client.docx.compiler import DocxCompiler
from euphorie.client.docx.compiler import DocxCompilerFrance
from euphorie.client.docx.compiler import DocxCompilerItaly
from euphorie.client.docx.compiler import IdentificationReportCompiler
from euphorie.client.interfaces import IFranceReportPhaseSkinLayer
from euphorie.client.interfaces import IItalyReportPhaseSkinLayer
from euphorie.client.session import SessionManager
from euphorie.client.utils import get_translated_custom_risks_title
from euphorie.content import MessageFactory as _
from euphorie.content.survey import get_tool_type
from plone.memoize.view import memoize
from Products.Five import BrowserView
from sqlalchemy import sql
from StringIO import StringIO
from urllib import quote
from z3c.saconfig import Session
from zope.i18n import translate
from collections import OrderedDict
from euphorie.client.docx.compiler import _sanitize_html
from json import loads


class OfficeDocumentView(BrowserView):
    ''' Base view that generates an office document and returns it
    '''
    _compiler = None
    _content_type = ''

    @property
    @memoize
    def session(self):
        ''' Return the session for this context/request
        '''
        return SessionManager.session

    def get_data(self, for_download=False):
        ''' Return the data for the compiler
        '''
        return {}

    def get_payload(self):
        ''' Compile the template and return the file as a string
        '''
        output = StringIO()
        compiler = self._compiler(self.context, self.request)
        compiler.compile(self.get_data(for_download=True))
        compiler.template.save(output)
        output.seek(0)
        return output.read()

    def t(self, txt):
        return translate(txt, context=self.request)

    def get_session_nodes(self):
        """ Return an ordered list of all relevant tree items for the current
            survey.
        """
        query = Session.query(model.SurveyTreeItem).filter(
            model.SurveyTreeItem.session == self.session).filter(
            sql.not_(model.SKIPPED_PARENTS)).filter(
                sql.or_(
                    model.MODULE_WITH_RISK_OR_TOP5_FILTER,
                    model.RISK_PRESENT_OR_TOP5_FILTER
                )
        ).order_by(model.SurveyTreeItem.path)

        return query.all()

    def get_modules(self):
        ''' Returns the modules for this session
        '''
        sql_modules = Session.query(model.Module).filter(
            model.SurveyTreeItem.session == self.session
        ).order_by(
            model.SurveyTreeItem.path
        )
        modules = []
        for sql_module in sql_modules:
            if sql_module.skip_children:
                continue
            if sql_module.zodb_path.find('custom-risks') != -1:
                module_title = get_translated_custom_risks_title(self.request)
            else:
                module_title = sql_module.title
            risks = self.get_risks_for(sql_module)
            modules.append({
                u'title': module_title,
                u'checked': bool(risks),
                u'risks': risks,
            })
        return modules

    def get_risks_for(self, sql_module):
        sql_risks = Session.query(model.Risk).filter(
            model.Risk.parent_id == sql_module.id
        ).order_by(
            model.SurveyTreeItem.path
        )
        risks = []
        for sql_risk in sql_risks:
            if sql_risk.identification == 'n/a':
                continue
            if not sql_risk.is_custom_risk:
                risk = self.request.survey.restrictedTraverse(
                    sql_risk.zodb_path.split("/"))
                risk_description = risk.description
                defined_measures = risk.get_pre_defined_measures(self.request)
            else:
                risk = None
                risk_description = defined_measures = ""
            risk_description = _sanitize_html(risk_description)
            try:
                # We try to get at least some order in: First, the pre-
                # defined measures that the user has confirmed, then the
                # additional custom-defined ones.
                saved_measures = loads(sql_risk.existing_measures)
                # Backwards compat. We used to save dicts in JSON before we
                # switched to list of tuples.
                if isinstance(saved_measures, dict):
                    saved_measures = [
                        (k, v) for (k, v) in saved_measures.items()]

                saved_measure_texts = OrderedDict()
                for text, on in saved_measures:
                    saved_measure_texts.update({text: on})

                existing_measures = []
                # Pick the pre-defined measures first
                for text in defined_measures:
                    active = saved_measure_texts.get(text)
                    if active is not None:
                        # Only add the measures that are active
                        if active:
                            existing_measures.append((text, 1))
                        saved_measure_texts.pop(text)

                # Finally, add the user-defined measures as well
                for text, on in saved_measure_texts.items():
                    existing_measures.append((text, on))

                measures = [item[0] for item in existing_measures if item[1]]
            except:
                measures = []
            if risk and (
                sql_risk.identification == 'no' or
                getattr(risk, 'type', None) == 'top5'
            ):
                actions = [
                    _get_action_plan(action)
                    for action in sql_risk.action_plans
                ]
            else:
                actions = []
            risks.append({
                u'title': sql_risk.title.strip(),
                u'description': risk_description,
                u'comment': _escape_text(sql_risk.comment),
                u'actions': actions,
                u'measures': measures,
                u'epilogue': u'',
                u'justifiable': sql_risk.identification,

            })
        return risks

    def __call__(self):
        self.request.response.setHeader(
            'Content-Disposition',
            'attachment; filename*=UTF-8\'\'{}'.format(
                quote(self._filename)
            ),
        )
        self.request.response.setHeader(
            "Content-Type",
            self._content_type,
        )
        return self.get_payload()


def _escape_text(txt):
    return txt and txt.replace('<', '&lt;') or ''


def _get_action_plan(action):
    action_plan = {}
    action_plan['text'] = _escape_text(action.action_plan)
    prevention_plan = getattr(action, 'prevention_plan') or ""
    action_plan['prevention_plan'] = _escape_text(prevention_plan)
    requirements = getattr(action, 'requirements') or ""
    action_plan['requirements'] = _escape_text(requirements)
    if action.responsible:
        action_plan['responsible'] = _escape_text(action.responsible)
    if action.planning_start:
        action_plan['planning_start'] = action.planning_start.strftime(
            "%d.%m.%Y")
    return action_plan


class ActionPlanDocxView(OfficeDocumentView):
    ''' Generate a report based on a basic docx template
    '''

    _compiler = DocxCompiler
    _content_type = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'  # noqa: E 501

    def __init__(self, context, request):
        super(ActionPlanDocxView, self).__init__(context, request)
        if IItalyReportPhaseSkinLayer.providedBy(request):
            self._compiler = DocxCompilerItaly
        elif IFranceReportPhaseSkinLayer.providedBy(request):
            if get_tool_type(context) == 'existing_measures':
                self._compiler = DocxCompilerFrance

    def get_heading(self, title):
        heading = self.t(
            _(
                "header_oira_report_download",
                default=u"OiRA Report: “${title}”",
                mapping=dict(title=title)
            ),
        )
        return heading

    def get_section_headings(self):
        headings = [
            self.t(_(
                "header_present_risks",
                default=u"Risks that have been identified, "
                u"evaluated and have an Action Plan")),
            self.t(_(
                "header_unevaluated_risks",
                default=u"Risks that have been identified but "
                u"do NOT have an Action Plan")),
            self.t(_(
                "header_unanswered_risks",
                default=u'Hazards/problems that have been "parked" '
                u'and are still to be dealt with')),
            self.t(_(
                "header_risks_not_present",
                default=u"Hazards/problems that have been managed "
                u"or are not present in your organisation"))
        ]
        return headings

    def get_sorted_nodes(self):
        nodes = self.get_session_nodes()
        session = SessionManager.session

        actioned_nodes = utils.get_actioned_nodes(nodes)
        unactioned_nodes = utils.get_unactioned_nodes(nodes)
        unanswered_nodes = utils.get_unanswered_nodes(session)
        risk_not_present_nodes = utils.get_risk_not_present_nodes(session)
        # From the non-present risks, filter out risks from the (un-)/actioned
        # categories. A "priority" risk will always appear in the action plan,
        # even if it has been answered with "Yes"
        risk_not_present_nodes = [
            n for n in risk_not_present_nodes if
            n not in actioned_nodes and n not in unactioned_nodes
        ]
        nodes = [
            actioned_nodes,
            unactioned_nodes,
            unanswered_nodes,
            risk_not_present_nodes,
        ]
        return nodes

    def get_data(self, for_download=False):
        ''' Gets the data structure in a format suitable for `DocxCompiler`
        '''

        data = {
            'title': self.session.title,
            'heading': self.get_heading(self.session.title),
            'section_headings': self.get_section_headings(),
            'nodes': self.get_sorted_nodes(),
            'survey_title': self.request.survey.title,
            'modules': self.get_modules()
        }

        return data

    @property
    def _filename(self):
        ''' Return the document filename
        '''
        filename = _(
            "filename_report_actionplan",
            default=u"Action plan ${title}",
            mapping={'title': self.session.title}
        )
        filename = translate(filename, context=self.request)
        return filename.encode('utf8') + '.docx'


class IdentificationReportDocxView(OfficeDocumentView):
    ''' Generate a report based on a basic docx template
    '''

    _compiler = IdentificationReportCompiler
    _content_type = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'  # noqa: E 501

    def get_session_nodes(self):
        """ Return an ordered list of all relevant tree items for the current
            survey.
        """
        query = Session.query(model.SurveyTreeItem).filter(
            model.SurveyTreeItem.session == self.session).filter(
                sql.not_(model.SKIPPED_PARENTS)
            ).order_by(model.SurveyTreeItem.path)

        return query.all()

    def get_data(self, for_download=False):
        ''' Gets the data structure in a format suitable for `DocxCompiler`
        '''

        data = {
            'title': self.session.title,
            'heading': '',
            'section_headings': [self.session.title],
            'nodes': [self.get_session_nodes()],
        }
        return data

    @property
    def _filename(self):
        ''' Return the document filename
        '''
        filename = _(
            "filename_report_identification",
            default=u"Identification report ${title}",
            mapping=dict(title=self.session.title))
        filename = translate(filename, context=self.request)
        return filename.encode('utf8') + '.docx'



# coding=utf-8
from euphorie.client.api.entry import API
from euphorie.client.api.users import Users
from euphorie.testing import EuphorieFunctionalTestCase

import json
import unittest


class APITests(unittest.TestCase):

    def test_get_known_entrypoint(self):
        api = API('api', 'request')
        api.entry_points = {'known': Users}
        child = api['known']
        self.assertTrue(isinstance(child, Users))
        self.assertEqual(child.request, 'request')
        self.assertEqual(child.getId(), 'known')


class BrowserAPITests(EuphorieFunctionalTestCase):

    def test_get_version(self):
        browser = self.get_browser()
        browser.open('http://nohost/plone/client/api')
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(
            set(response), set(['api-version', 'euphorie-version'])
        )
        self.assertEqual(response['api-version'], [1, 0])



# -*- coding: UTF-8 -*-
from euphorie.client import MessageFactory as _
from euphorie.client.country import IClientCountry
from euphorie.client.publish import EnableCustomRisks
from euphorie.client.sector import IClientSector
from plone import api
from z3c.appconfig.interfaces import IAppConfig
from z3c.appconfig.utils import asBool
import logging
import zope.component
import datetime

log = logging.getLogger(__name__)


def update_custom_risks_module_texts(context):
    """ """
    appconfig = zope.component.getUtility(IAppConfig)
    if not asBool(appconfig["euphorie"].get("allow_user_defined_risks")):
        log.warning(
            "Custom risks are not enabled. Set 'allow_user_defined_risks' to "
            "true in euphorie.ini for enabling them.")
        return
    portal = api.portal.get()
    client = portal.client
    count = 0
    for country in client.objectValues():
        if IClientCountry.providedBy(country):
            for sector in country.objectValues():
                if IClientSector.providedBy(sector):
                    for survey in sector.objectValues():

                        is_new = EnableCustomRisks(survey)
                        count += 1
                        custom = getattr(survey, 'custom-risks', None)
                        if custom:
                            # custom.title = _(u'title_other_risks', default=u"Added risks (by you)")
                            # custom.description = _(
                            #     u"description_other_risks",
                            #     default=u"In case you have identified risks not included in "
                            #     u"the tool, you are able to add them now:")
                            custom.question = _(
                                u"question_other_risks",
                                default=u"<p><strong>Important:</strong> In "
                                u"order to avoid duplicating risks, we strongly recommend you "
                                u"to go first through all the previous modules, if you have not "
                                u"done it yet.</p><p>If you don't need to add risks, please continue.</p>")
                        if is_new:
                            survey.published = (
                                survey.id, survey.title, datetime.datetime.now())
                        # except Exception as e:
                        #     log.error("Could not enable custom risks for module. %s" % e)



# coding=utf-8
from euphorie.client.api.actionplan import plan_info
from euphorie.client.api.actionplan import View
from euphorie.client.api.authentication import generate_token
from euphorie.client.api.tests.utils import _setup_session
from euphorie.client.model import ActionPlan
from euphorie.client.model import Risk
from euphorie.client.model import Session
from euphorie.testing import EuphorieFunctionalTestCase
from plone import api

import datetime
import json
import mock
import unittest


class plan_info_tests(unittest.TestCase):
    def plan_info(self, *a, **kw):
        return plan_info(*a, **kw)

    def test_minimal(self):
        plan = ActionPlan(id=15, action_plan=u'This is the plan')
        info = self.plan_info(plan)
        self.assertEqual(
                set(info),
                set(['id', 'plan', 'prevention', 'requirements', 'responsible',
                    'budget', 'planning-start', 'planning-end', 'reference']))
        self.assertEqual(info['plan'], u'This is the plan')
        self.assertEqual(info['prevention'], None)
        self.assertEqual(info['requirements'], None)
        self.assertEqual(info['responsible'], None)
        self.assertEqual(info['budget'], None)
        self.assertEqual(info['planning-start'], None)
        self.assertEqual(info['planning-end'], None)

    def test_with_dates(self):
        plan = ActionPlan(
                planning_start=datetime.date(2012, 6, 3),
                planning_end=datetime.date(2012, 6, 4))
        info = self.plan_info(plan)
        self.assertEqual(info['planning-start'], '2012-06-03')
        self.assertEqual(info['planning-end'], '2012-06-04')


class ViewTests(unittest.TestCase):

    def test_do_GET_use_plan_info(self):
        view = View('context', None)
        with mock.patch('euphorie.client.api.actionplan.plan_info') \
                as mock_plan_info:
            mock_plan_info.return_value = {'data': 'mock'}
            self.assertEqual(
                    view.do_GET(),
                    {'type': 'actionplan', 'data': 'mock'})
            mock_plan_info.assert_called_once_with('context')

    def test_do_PUT_returns_info(self):
        view = View(ActionPlan(), None)
        view.input = {}
        view.do_GET = mock.Mock(return_value='info')
        self.assertEqual(view.do_PUT(), 'info')

    def test_do_PUT_update_data(self):
        context = ActionPlan()
        view = View(context, None)
        view.input = {'plan': 'Collect underpants',
                      'budget': 15,
                      'planning-start': '2012-06-04'}
        view.do_PUT()
        self.assertEqual(context.action_plan, u'Collect underpants')
        self.assertEqual(context.budget, 15)
        self.assertEqual(context.planning_start, datetime.date(2012, 6, 4))

    def test_do_PUT_bad_data(self):
        context = ActionPlan()
        view = View(context, None)
        view.input = {'budget': 'unlimited'}
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')


class BrowserTests(EuphorieFunctionalTestCase):

    def test_get(self):
        with api.env.adopt_user('admin'):
            (account, survey, survey_session) = _setup_session(self.portal)
            from transaction import commit
            commit()
            risk = Session.query(Risk).one()
            risk.action_plans.append(ActionPlan(
                action_plan=u'This is the plan',
                planning_start=datetime.date(2012, 6, 3))
            )
        browser = self.get_browser()
        browser.addHeader('X-Euphorie-Token', generate_token(account))
        browser.open('http://nohost/plone/client/api/users/1/'
                     'sessions/1/1/1/actionplans/1')
        self.assertEqual(
            browser.headers['Content-Type'], 'application/json'
        )
        response = json.loads(browser.contents)
        self.assertEqual(response['plan'], u'This is the plan')



# coding=utf-8
from euphorie.client import model
from euphorie.client.authentication import EuphorieAccountPlugin
from euphorie.client.interfaces import IClientSkinLayer
from euphorie.client.tests.database import DatabaseTests
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin  # noqa
from Products.PluggableAuthService.interfaces.plugins import IChallengePlugin
from Products.PluggableAuthService.interfaces.plugins import IExtractionPlugin
from Products.PluggableAuthService.interfaces.plugins import IUserEnumerationPlugin  # noqa
from Products.PluggableAuthService.interfaces.plugins import IUserFactoryPlugin
from z3c.saconfig import Session
from zope.interface import directlyProvides
from zope.interface.verify import verifyClass

import mock


class MockContext(object):

    def absolute_url(self):
        return 'http://www.example.com/base'


class MockRequest(object):
    PUBLISHED = MockContext()

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def get(self, key, default=None):
        return getattr(self, key, default)

    getHeader = get


class MockResponse(object):

    def redirect(self, url, lock):
        self.redirect_url = url
        self.redirect_lock = lock


class EuphorieAccountPluginTests(DatabaseTests):

    def test_extraction_interface(self):
        verifyClass(IExtractionPlugin, EuphorieAccountPlugin)

    def test_extraction_no_token_header(self):
        plugin = EuphorieAccountPlugin('plugin')
        request = mock.Mock()
        request.getHeader.return_value = None
        self.assertEqual(plugin.extractCredentials(request), {})

    def test_extraction_with_token_header(self):
        plugin = EuphorieAccountPlugin('plugin')
        request = mock.Mock()
        request.getHeader.return_value = 'the-token'
        self.assertEqual(
            plugin.extractCredentials(request), {
                'api-token': 'the-token'
            }
        )

    def test_authenticate_interface(self):
        verifyClass(IAuthenticationPlugin, EuphorieAccountPlugin)

    def test_authenticate_token_no_token(self):
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(plugin._authenticate_token({}), None)

    def test_authenticate_token_call_client_authenticate(self):
        plugin = EuphorieAccountPlugin('plugin')
        with mock.patch(
            'euphorie.client.authentication.authenticate_client_token',
            return_value='result'
        ):
            self.assertEqual(
                plugin._authenticate_token({
                    'api-token': 'y'
                }), 'result'
            )

    def test_authenticate_token_call_cms_authenticate(self):
        plugin = EuphorieAccountPlugin('plugin')
        with mock.patch(
            'euphorie.client.authentication.authenticate_cms_token',
            return_value='result'
        ):
            self.assertEqual(
                plugin._authenticate_token({
                    'api-token': 'y'
                }), 'result'
            )

    def test_authenticate_login_wrong_credential_type(self):
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(
            plugin._authenticate_login({
                'cookie': 'yummie'
            }), None
        )

    def test_authenticate_login_unknown_account(self):
        plugin = EuphorieAccountPlugin('plugin')
        credentials = {'login': 'login', 'password': u'secret'}
        self.assertEqual(plugin._authenticate_login(credentials), None)

    def test_authenticate_login_valid_login(self):
        session = Session()
        account = model.Account(loginname='john', password=u'jane')
        session.add(account)
        plugin = EuphorieAccountPlugin('plugin')
        credentials = {'login': 'john', 'password': u'jane'}
        self.assertTrue(plugin._authenticate_login(credentials) is not None)

    def test_authenticate_login_not_case_sensitive(self):
        session = Session()
        account = model.Account(loginname='john', password=u'jane')
        session.add(account)
        plugin = EuphorieAccountPlugin('plugin')
        credentials = {'login': 'JoHn', 'password': u'jane'}
        self.assertTrue(plugin._authenticate_login(credentials) is not None)

    def test_CreateUser_interface(self):
        verifyClass(IUserFactoryPlugin, EuphorieAccountPlugin)

    def test_CreateUser_unknown_account(self):
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(plugin.createUser('1', 'john'), None)

    def testCreateUser_ValidAccount(self):
        session = Session()
        account = model.Account(loginname='john', password=u'jane')
        session.add(account)
        plugin = EuphorieAccountPlugin('plugin')
        self.assertTrue(plugin.createUser('1', 'john') is account)

    def testEnumerateUsers_Interface(self):
        verifyClass(IUserEnumerationPlugin, EuphorieAccountPlugin)

    def testEnumerateUsers_NoInexactMatch(self):
        session = Session()
        account = model.Account(loginname='john', password=u'jane')
        session.add(account)
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(
            plugin.enumerateUsers(login='john', exact_match=False), []
        )

    def test_EnumerateUsers_search_by_id(self):
        session = Session()
        account = model.Account(loginname='john', password=u'jane')
        session.add(account)
        request = MockRequest(ACTUAL_URL='http://www.example.com/client')
        directlyProvides(request, IClientSkinLayer)
        plugin = EuphorieAccountPlugin('plugin')
        plugin.REQUEST = request
        info = plugin.enumerateUsers(id='1', exact_match=True)
        self.assertEqual(info, [{'id': '1', 'login': 'john'}])
        self.assertTrue(isinstance(info[0]['id'], str))
        self.assertTrue(isinstance(info[0]['login'], str))

    def test_EnumerateUsers_search_by_login(self):
        session = Session()
        account = model.Account(loginname='john', password=u'jane')
        session.add(account)
        request = MockRequest(ACTUAL_URL='http://www.example.com/client')
        directlyProvides(request, IClientSkinLayer)
        plugin = EuphorieAccountPlugin('plugin')
        plugin.REQUEST = request
        self.assertEqual(
            plugin.enumerateUsers(login='john', exact_match=True), [{
                'id': '1',
                'login': 'john'
            }]
        )

    def test_EnumerateUsers_search_by_login_and_id(self):
        session = Session()
        account = model.Account(loginname='john', password=u'jane')
        session.add(account)
        request = MockRequest(ACTUAL_URL='http://www.example.com/client')
        directlyProvides(request, IClientSkinLayer)
        plugin = EuphorieAccountPlugin('plugin')
        plugin.REQUEST = request
        self.assertEqual(
            plugin.enumerateUsers(id='1', login='john', exact_match=True), [{
                'id': '1',
                'login': 'john'
            }]
        )

    def test_EnumerateUsers_unknown_account(self):
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(plugin.enumerateUsers(id='1', exact_match=False), [])

    def test_Challenge_interface(self):
        verifyClass(IChallengePlugin, EuphorieAccountPlugin)

    def test_Challenge_require_IClientSkinLayer(self):
        request = MockRequest(ACTUAL_URL='http://www.example.com/client')
        response = MockResponse()
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(plugin.challenge(request, response), False)

    def test_Challenge_no_query_string(self):
        request = MockRequest(ACTUAL_URL='http://www.example.com/client')
        directlyProvides(request, IClientSkinLayer)
        response = MockResponse()
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(plugin.challenge(request, response), True)
        self.assertEqual(
            response.redirect_url, 'http://www.example.com/base/@@login?'
            'came_from=http%3A%2F%2Fwww.example.com%2Fclient'
        )
        self.assertEqual(bool(response.redirect_lock), True)

    def test_Challenge_with_query_string(self):
        request = MockRequest(
            ACTUAL_URL='http://www.example.com/client', QUERY_STRING='one=1'
        )
        directlyProvides(request, IClientSkinLayer)
        response = MockResponse()
        plugin = EuphorieAccountPlugin('plugin')
        self.assertEqual(plugin.challenge(request, response), True)
        self.assertEqual(
            response.redirect_url, 'http://www.example.com/base/@@login?'
            'came_from=http%3A%2F%2Fwww.example.com%2Fclient%3Fone%3D1'
        )



# coding=utf-8
from ..json import export_image
from ..json import get_json_bool
from ..json import get_json_date
from ..json import get_json_int
from ..json import get_json_string
from ..json import get_json_token
from ..json import JsonView
from euphorie.testing import EuphorieFunctionalTestCase
from plone.namedfile.file import NamedBlobImage
from z3c.appconfig.interfaces import IAppConfig
from zope.component import getUtility
from zope.interface import Interface
from zope.publisher.browser import TestRequest
from zope.schema import Choice
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import unittest


class get_json_token_tests(unittest.TestCase):

    def get_json_token(self, *a, **kw):
        return get_json_token(*a, **kw)

    def test_missing_required_field(self):
        self.assertRaises(
            KeyError, self.get_json_token, {}, 'field', None, required=True
        )

    def test_missing_optional_field(self):
        self.assertEqual(
            self.get_json_token({},
                                'field',
                                None,
                                required=False,
                                default='default'), 'default'
        )

    def test_bad_value(self):
        self.assertRaises(
            ValueError, self.get_json_token, {'field': 'oops'}, 'field',
            DummySchema['field']
        )

    def test_correct_value(self):
        self.assertEqual(
            self.get_json_token({
                'field': u'foo'
            }, 'field', DummySchema['field']), u'foo'
        )


class get_json_string_tests(unittest.TestCase):

    def get_json_string(self, *a, **kw):
        return get_json_string(*a, **kw)

    def test_missing_required_field(self):
        self.assertRaises(
            KeyError, self.get_json_string, {}, 'field', required=True
        )

    def test_missing_optional_field(self):
        self.assertEqual(
            self.get_json_string({},
                                 'field',
                                 required=False,
                                 default='default'), 'default'
        )

    def test_bad_type(self):
        self.assertRaises(
            ValueError, self.get_json_string,
            {'field': False}, 'field', DummySchema['field']
        )

    def test_proper_value(self):
        self.assertEqual(
            self.get_json_string({
                'field': 'value'
            }, 'field'), 'value'
        )

    def test_trim_length(self):
        self.assertEqual(
            self.get_json_string({
                'field': 'value'
            }, 'field', length=2), 'va'
        )


class get_json_bool_tests(unittest.TestCase):

    def get_json_bool(self, *a, **kw):
        return get_json_bool(*a, **kw)

    def test_missing_required_field(self):
        self.assertRaises(
            KeyError, self.get_json_bool, {}, 'field', required=True
        )

    def test_missing_optional_field(self):
        self.assertEqual(
            self.get_json_bool({}, 'field', required=False, default='default'),
            'default'
        )

    def test_bad_type(self):
        self.assertRaises(
            ValueError, self.get_json_bool, {'field': 'dummy'}, 'field',
            DummySchema['field']
        )

    def test_proper_value(self):
        self.assertEqual(self.get_json_bool({'field': True}, 'field'), True)


class get_json_int_tests(unittest.TestCase):

    def get_json_int(self, *a, **kw):
        return get_json_int(*a, **kw)

    def test_missing_required_field(self):
        self.assertRaises(
            KeyError, self.get_json_int, {}, 'field', required=True
        )

    def test_missing_optional_field(self):
        self.assertEqual(
            self.get_json_int({}, 'field', required=False, default='default'),
            'default'
        )

    def test_bad_type(self):
        self.assertRaises(
            ValueError, self.get_json_int, {'field': 'dummy'}, 'field',
            DummySchema['field']
        )

    def test_proper_value(self):
        self.assertEqual(self.get_json_int({'field': 15}, 'field'), 15)


class get_json_date_tests(unittest.TestCase):

    def get_json_date(self, *a, **kw):
        return get_json_date(*a, **kw)

    def test_missing_required_field(self):
        self.assertRaises(
            KeyError, self.get_json_date, {}, 'field', required=True
        )

    def test_missing_optional_field(self):
        self.assertEqual(
            self.get_json_date({}, 'field', required=False, default='default'),
            'default'
        )

    def test_bad_type(self):
        self.assertRaises(
            ValueError, self.get_json_date, {'field': 15}, 'field',
            DummySchema['field']
        )

    def test_invalid_date(self):
        self.assertRaises(
            ValueError, self.get_json_date, {'field': '2012-06-four'}, 'field',
            DummySchema['field']
        )

    def test_proper_value(self):
        import datetime
        self.assertEqual(
            self.get_json_date({
                'field': '2012-06-04'
            }, 'field'), datetime.date(2012, 6, 4)
        )


class export_image_tests(EuphorieFunctionalTestCase):

    def export_image(self, *a, **kw):
        return export_image(*a, **kw)

    def setup_context(self):
        self.loginAsPortalOwner()
        self.portal.sectors.nl.invokeFactory(
            'euphorie.sector', 'dining', title=u'Fine dining'
        )
        sector = self.portal.sectors.nl.dining
        sector.invokeFactory('euphorie.surveygroup', 'survey', title=u'Survey')
        surveygroup = sector.survey
        surveygroup.invokeFactory(
            'euphorie.survey', 'version1', title=u'Version 1'
        )
        survey = surveygroup.version1
        survey.invokeFactory('euphorie.module', '1', title=u'Module 1')
        module = survey['1']
        gif = 'GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\xff' \
               '\xff\xff!\xf9\x04\x01\x00\x00\x01\x00,\x00\x00\x00' \
               '\x00\x01\x00\x01\x00\x00\x02\x01L\x00;'  # noqa
        module.image = NamedBlobImage(
            data=gif, contentType='image/gif', filename=u'pixel.gif'
        )
        module.caption = u'Caption'
        view = survey.restrictedTraverse('@@publish')
        view.publish()
        return self.portal.client['nl']['dining']['survey']['1']

    def test_unknown_image(self):
        context = self.setup_context()
        context.image = None
        request = TestRequest()
        response = self.export_image(
            context, request, 'image', u'Caption', width=100, height=100
        )
        self.assertEqual(response, None)

    def test_standard(self):
        config = getUtility(IAppConfig)
        del config['euphorie']['client']
        context = self.setup_context()
        request = TestRequest()
        request.client = self.portal.client
        response = self.export_image(
            context, request, 'image', 'caption', width=100, height=100
        )
        self.assertEqual(
            set(response), set(['thumbnail', 'original', 'caption'])
        )
        self.assertTrue(
            response['thumbnail'].startswith('http://nohost/plone/client/nl')
        )
        self.assertTrue(
            response['original'].startswith('http://nohost/plone/client/nl')
        )
        self.assertEqual(response['caption'], u'Caption')


class JsonViewTests(unittest.TestCase):

    def JsonView(self, *a, **kw):
        return JsonView(*a, **kw)

    def test_invalid_input(self):
        import json
        import StringIO
        request = TestRequest()
        request.stdin = StringIO.StringIO('invalid json')
        view = self.JsonView(None, request)
        response = view()
        self.assertEqual(
            json.loads(response), {
                'type': 'error',
                'message': 'Invalid JSON input'
            }
        )

    def test_bad_method(self):
        import json
        import StringIO
        import mock
        request = TestRequest()
        request.stdin = StringIO.StringIO()
        context = mock.Mock()
        context.getPhysicalPath.return_value = ['', 'Plone', 'client', 'api']
        view = self.JsonView(context, request)
        response = view()
        self.assertEqual(request.response.getStatus(), 405)
        self.assertEqual(
            json.loads(response), {
                'type': 'error',
                'message': 'HTTP method not allowed'
            }
        )

    def test_do_OPTIONS_minimal(self):
        request = TestRequest()
        view = self.JsonView(None, request)
        view.do_OPTIONS()
        self.assertEqual(request.response.getHeader('Allow'), 'OPTIONS')

    def test_do_OPTIONS_with_GET_handler(self):
        request = TestRequest()
        view = self.JsonView(None, request)
        view.do_GET = lambda: None
        view.do_OPTIONS()
        self.assertEqual(request.response.getHeader('Allow'), 'GET,OPTIONS')


class DummySchema(Interface):
    field = Choice(
        vocabulary=SimpleVocabulary([SimpleTerm(u'foo', title=u'Bar')])
    )



# coding=utf-8
from euphorie.client.api.authentication import generate_token
from euphorie.client.api.profile import View
from euphorie.client.model import SurveySession
from euphorie.client.tests.utils import addAccount
from euphorie.client.tests.utils import addSurvey
from euphorie.content.profilequestion import ProfileQuestion
from euphorie.content.survey import Survey
from euphorie.content.tests.utils import PROFILE_SURVEY
from euphorie.testing import EuphorieFunctionalTestCase
from euphorie.testing import EuphorieIntegrationTestCase
from z3c.saconfig import Session

import datetime
import json
import mock


class ViewTests(EuphorieIntegrationTestCase):

    def View(self, *a, **kw):
        return View(*a, **kw)

    def test_get_no_profile(self):
        survey = Survey(id='5')
        view = self.View(survey, None)
        view.survey = mock.Mock(return_value=survey)
        with mock.patch('euphorie.client.api.profile.extractProfile') \
                as mock_extractProfile:
            mock_extractProfile.return_value = {}
            response = view.do_GET()
            self.assertEqual(response['profile'], [])

    def test_get_with_profile(self):
        self.portal.survey = Survey(id='survey')
        survey = self.portal.survey
        survey._setOb('5', ProfileQuestion(id='5', question=u'Locations'))
        view = self.View(survey, None)
        view.survey = mock.Mock(return_value=survey)
        with mock.patch('euphorie.client.api.profile.extractProfile') \
                as mock_extractProfile:
            mock_extractProfile.return_value = {'5': [u'London', u'Tokyo']}
            response = view.do_GET()
            profile = response['profile']
            self.assertEqual(
                profile, [{
                    'id': '5',
                    'question': u'Locations',
                    'value': [u'London', u'Tokyo']
                }]
            )

    def test_put_no_profile(self):
        account = addAccount(password='secret')
        survey_session = SurveySession(
            title=u'Dummy session',
            zodb_path='nl/ict/software-development',
            account=account
        )
        view = self.View(survey_session, None)
        view.input = {}
        Session.add(survey_session)
        survey = Survey(id='survey')
        view.survey = mock.Mock(return_value=survey)
        with mock.patch('euphorie.client.api.profile.extractProfile') \
                as mock_extractProfile:
            mock_extractProfile.return_value = {}
            response = view.do_PUT()
            self.assertEqual(response['profile'], [])

    def test_put_too_much_data(self):
        import mock
        from euphorie.client.tests.utils import addAccount
        from euphorie.client.model import SurveySession
        from euphorie.content.survey import Survey
        account = addAccount(password='secret')
        survey_session = SurveySession(
            title=u'Dummy session',
            zodb_path='nl/ict/software-development',
            account=account
        )
        view = self.View(survey_session, None)
        view.input = {'5': True}
        survey = Survey(id='survey')
        view.survey = mock.Mock(return_value=survey)
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')

    def test_put_not_all_questions_answered(self):
        import mock
        from euphorie.content.survey import Survey
        from euphorie.content.profilequestion import ProfileQuestion
        from euphorie.client.tests.utils import addAccount
        from euphorie.client.model import SurveySession
        account = addAccount(password='secret')
        survey_session = SurveySession(
            title=u'Dummy session',
            zodb_path='nl/ict/software-development',
            account=account
        )
        view = self.View(survey_session, None)
        view.input = {}
        self.portal.survey = Survey(id='survey')
        survey = self.portal.survey
        survey._setOb('5', ProfileQuestion(id='5', question=u'Locations'))
        view.survey = mock.Mock(return_value=survey)
        response = view.do_PUT()
        self.assertEqual(response['type'], 'error')


class BrowserTests(EuphorieFunctionalTestCase):

    def test_get_empty_profile(self):
        self.loginAsPortalOwner()
        addSurvey(self.portal, PROFILE_SURVEY)
        account = addAccount(password='secret')
        survey_session = SurveySession(
            title=u'Dummy session',
            created=datetime.datetime(2012, 4, 22, 23, 5, 12),
            modified=datetime.datetime(2012, 4, 23, 11, 50, 30),
            zodb_path='nl/ict/software-development',
            account=account
        )
        Session.add(survey_session)
        browser = self.get_browser()
        browser.addHeader('X-Euphorie-Token', generate_token(account))
        browser.open(
            'http://nohost/plone/client/api/users/1/sessions/1/profile'
        )
        self.assertEqual(browser.headers['Content-Type'], 'application/json')
        response = json.loads(browser.contents)
        self.assertEqual(
            set(response), set(['id', 'type', 'title', 'profile'])
        )
        self.assertEqual(response['id'], 1)
        self.assertEqual(response['type'], 'profile')
        self.assertEqual(response['title'], u'Dummy session')
        self.assertEqual(
            response['profile'], [{
                'id': u'1',
                'question': u'List all your departments:',
                'value': []
            }]
        )



# coding=utf-8
import docx
import htmllaundry
import lxml.html
from lxml import etree


def add_hyperlink_into_run(paragraph, run, url):
    runs = paragraph.runs
    i = 0
    for i in range(len(runs)):
        if runs[i].text == run.text:
            break

    # This gets access to the document.xml.rels file and gets a new
    # relation id value
    part = paragraph.part
    r_id = part.relate_to(
        url,
        docx.opc.constants.RELATIONSHIP_TYPE.HYPERLINK,
        is_external=True)

    # Create the w:hyperlink tag and add needed values
    hyperlink = docx.oxml.shared.OxmlElement('w:hyperlink')
    hyperlink.set(docx.oxml.shared.qn('r:id'), r_id, )
    hyperlink.append(run._r)
    paragraph._p.insert(i + 1, hyperlink)


class _HtmlToWord(object):

    def handleInlineText(self, node, p, run=None):
        """Handler for elements which can only contain inline text (p, li)"""
        if not run:
            run = p.add_run()
        font = run.font
        if node.tag in ["strong", "b"]:
            font.bold = True
        elif node.tag in ["em", "i"]:
            font.italic = True
        elif node.tag == "u":
            font.underline = True

        if node.text and node.text.strip():
            keep_run = False
            if node.tag == 'a':
                href = node.get('href')
                href = href and href.strip()
                if href and href != node.text.strip():
                    run.style = "Hyperlink"
                    run.text = node.text
                    add_hyperlink_into_run(p, run, href)
                else:
                    run.text = node.text
            else:
                run.text = node.text
        else:
            keep_run = True

        for sub in node:
            p = self.handleInlineText(sub, p, run=keep_run and run)
        if node.tail and node.tail.strip():
            run = p.add_run()
            run.text = node.tail
        return p

    def handleElement(self, node, doc, style=None):
        if node.tag in ["p", "li", 'strong', 'b', 'em', 'i', 'u', 'a']:
            p = doc.add_paragraph(style=style)
            p = self.handleInlineText(node, p)
        elif node.tag in ["ul", "ol"]:

            if node.tag == "ul":
                style = "List Bullet"
            else:
                style = "List Number"
            for sub in node:
                if sub.tag == "li":
                    p = doc.add_paragraph(style=style)
                    p = self.handleInlineText(sub, p)

        tail = node.tail
        # Prevent unwanted empty lines inside listings and paragraphs that come
        # from newlines in the markup
        # if node.tag in ['li', 'p', 'strong', 'em', 'b', 'i']:
        tail = tail and tail.strip()
        if tail:
            doc.add_paragraph(tail)
        return doc

    def __call__(self, markup, doc, style=None):
        if not markup or not markup.strip():
            return doc
        try:
            markup_doc = lxml.html.document_fromstring(markup)
        except etree.XMLSyntaxError:
            text = htmllaundry.StripMarkup(markup)
            text = text.replace("&#13", "\n")
            doc.add_paragraph(text)
            return doc

        for node in markup_doc.find('body'):
            doc = self.handleElement(node, doc, style)

        return doc


HtmlToWord = _HtmlToWord()



# coding=utf-8
from AccessControl import getSecurityManager
from Acquisition import aq_base
from Acquisition import aq_chain
from Acquisition import aq_inner
from Acquisition import aq_parent
from datetime import datetime
from euphorie import MessageFactory as _
from euphorie.client import config
from euphorie.client.client import IClient
from euphorie.client.cookie import setCookie
from euphorie.client.country import IClientCountry
from euphorie.client.interfaces import IItaly
from euphorie.client.model import get_current_account
from euphorie.client.sector import IClientSector
from euphorie.client.session import SESSION_COOKIE
from euphorie.client.session import SessionManager
from euphorie.client.utils import getSecret
from euphorie.content.survey import ISurvey
from euphorie.content.utils import StripMarkup
from euphorie.decorators import reify
from euphorie.ghost import PathGhost
from json import dumps
from logging import getLogger
from os import path
from plone import api
from plone.i18n.normalizer import idnormalizer
from plone.memoize.instance import memoize
from plone.memoize.view import memoize_contextless
from plonetheme.nuplone.utils import isAnonymous
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from z3c.appconfig.interfaces import IAppConfig
from z3c.appconfig.utils import asBool
from ZODB.POSException import POSKeyError
from zope.browser.interfaces import IBrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.hooks import getSite
from zope.i18nmessageid import MessageFactory

import Globals


pl_message = MessageFactory("plonelocales")
pae_message = MessageFactory("plone.app.event")
logger = getLogger(__name__)


# XXX should to to config (registry?)
NAME_TO_PHASE = {
    'start': 'preparation',
    'profile': 'preparation',
    'identification': 'identification',
    'customization': 'identification',
    'actionplan': 'actionplan',
    'report': 'report',
    'status': 'status',
    'help': 'help',
    'new-email': 'useraction',
    'account-settings': 'useraction',
    'account-delete': 'useraction',
    'update': 'preparation',
    'disclaimer': 'help',
    'terms-and-conditions': 'help',
    'training': 'training',
}


class WebHelpers(BrowserView):
    """Browser view with utility methods that can be used in templates.

    Several methods in this view assume that the current survey can be
    found as an attribute on the request. This is normally setup by the
    :py:class:`euphorie.client.survey.SurveyPublishTraverser` traverser.

    View name: @@webhelpers
    """
    sector = None
    SESSION_COOKIE = SESSION_COOKIE

    resources_name = "++resource++euphorie.resources"
    js_resources_name = "++resource++euphorie.resources"
    bundle_name = "bundle.js"
    bundle_name_min = "bundle.min.js"

    def __init__(self, context, request):
        super(WebHelpers, self).__init__(context, request)
        if self.anonymous:
            setattr(self.request, 'survey', self._tool)

    @property
    @memoize
    def _my_context(self):
        context = self.context
        if IBrowserView.providedBy(context):
            context = context.context
        return context

    @property
    @memoize
    def sector(self):
        for obj in aq_chain(aq_inner(self._my_context)):
            if IClientSector.providedBy(obj):
                return obj

    @property
    @memoize
    def debug_mode(self):
        return Globals.DevelopmentMode

    @property
    @memoize
    def _settings(self):
        appconfig = getUtility(IAppConfig)
        return appconfig.get('euphorie')

    @property
    @memoize
    def allow_social_sharing(self):
        return asBool(self._settings.get('allow_social_sharing', False))

    @property
    @memoize
    def allow_guest_accounts(self):
        return asBool(self._settings.get('allow_guest_accounts', False))

    @property
    @memoize
    def use_training_module(self):
        return asBool(self._settings.get('use_training_module', False))

    @property
    @memoize
    def use_publication_feature(self):
        return asBool(self._settings.get('use_publication_feature', False))

    @property
    def use_clone_feature(self):
        return asBool(self._settings.get('use_clone_feature', False))

    @property
    @memoize
    def default_country(self):
        return self._settings.get('default_country', '')

    @property
    @memoize
    def _user(self):
        return getSecurityManager().getUser()

    @property
    @memoize
    def anonymous(self):
        return isAnonymous(self._user)

    @property
    @memoize
    def is_guest_account(self):
        account = getattr(self._user, 'account_type', None)
        return account == config.GUEST_ACCOUNT

    @property
    @memoize
    def session(self):
        return getattr(SessionManager, 'session', None)

    @property
    @memoize
    def is_new_session(self):
        if self.request.get('new_session'):
            return True
        if self.session is not None:
            return self.session.children().count() == 0
        return False

    @property
    @memoize
    def guest_session_id(self):
        return self.is_guest_account and self.session_id or None

    @property
    @memoize
    def session_id(self):
        return getattr(self.session, 'id', '')

    @memoize
    def session_by_id(self, sessionid):
        return SessionManager.get_session_by_id(sessionid)

    @property
    @memoize
    def came_from(self):
        came_from = self.request.form.get("came_from")
        if not came_from:
            return aq_parent(self._my_context).absolute_url()
        if not isinstance(came_from, list):
            return came_from
        # If came_from is both in the querystring and the form data
        return came_from[0]

    @property
    @memoize
    def country_name(self):
        for obj in aq_chain(aq_inner(self._my_context)):
            if IClientCountry.providedBy(obj):
                return obj.Title()

    @property
    @memoize
    def sector_name(self):
        for obj in aq_chain(aq_inner(self._my_context)):
            if IClientSector.providedBy(obj):
                return obj.Title()

    @property
    @memoize
    def _tool(self):
        for obj in aq_chain(aq_inner(self._my_context)):
            if ISurvey.providedBy(obj):
                return obj

    @property
    @memoize
    def tool_name(self):
        obj = self._tool
        if not obj:
            return ''
        return obj.Title()

    @property
    @memoize
    def tool_description(self):
        obj = self._tool
        if not obj:
            return ''
        ploneview = self._my_context.restrictedTraverse('@@plone')
        return ploneview.cropText(StripMarkup(obj.introduction), 800)

    @property
    @memoize
    def language_code(self):
        lt = getToolByName(self._my_context, 'portal_languages')
        lang = lt.getPreferredLanguage()
        return lang

    def get_username(self):
        member = api.user.get_current()
        return member.getProperty('fullname') or member.getUserName()

    def get_webstats_js(self):
        return api.portal.get_registry_record('plone.webstats_js')

    def language_dict(self):
        site = getSite()
        ltool = getToolByName(site, 'portal_languages')
        return ltool.getAvailableLanguages()

    @property
    def macros(self):
        return self.index.macros

    def country(self):
        for obj in aq_chain(aq_inner(self._my_context)):
            if IClientCountry.providedBy(obj):
                return obj.id
        return None

    def logoMode(self):
        return 'alien' if 'alien' in self.extra_css else 'native'

    @reify
    def styles_override(self):
        css = ""
        if IItaly.providedBy(self.request):
            css = """
#steps .topics .legend li.answered.risk::before {
    background: purple;
}
#steps .topics .questions li.answered.risk a::before {
    background: purple;
}
"""
        return css

    @reify
    def extra_css(self):
        sector = self.sector
        if sector is None:
            return u''

        sector = aq_base(sector)
        parts = []

        main_background = getattr(sector, 'main_background_colour', None)
        main_foreground = getattr(sector, 'main_foreground_colour', None)
        support_background = getattr(sector, 'support_background_colour', None)
        support_foreground = getattr(sector, 'support_foreground_colour', None)
        if main_background and main_foreground and \
                support_background and support_foreground:
            parts.append('deCornae')
            parts.append(
                    'brightMainColour'
                    if sector.main_background_bright
                    else 'darkMainColour')
            parts.append(
                    'brightSupportColour'
                    if sector.support_background_bright
                    else 'darkSupportColour')

        if getattr(sector, 'logo', None) is not None:
            parts.append('alien')

        lt = getToolByName(self._my_context, 'portal_languages')
        lang = lt.getPreferredLanguage()
        parts.append('language-%s' % lang)

        return ' ' + ' '.join(parts)

    @reify
    def sector_title(self):
        """Return the title to use for the current sector. If the current
        context is not in a sector return the agency name instead.
        """
        sector = self.sector
        if sector is not None and \
                getattr(aq_base(sector), 'logo', None) is not None:
            return sector.Title()
        else:
            return _(
                'title_tool',
                default=u'OiRA - Online interactive Risk Assessment',
            )

    @reify
    def client_url(self):
        """Return the absolute URL for the client."""
        return self.request.client.absolute_url()

    @reify
    def country_or_client_url(self):
        """Return the country URL, but fall back to the client URL in case
        the country URL is None.
        Relevant in tests only, as far as I can see"""
        return self.country_url or self.client_url

    def _base_url(self):
        """Return a base URL to be used for non-survey specific pages.
        If we are in a survey the help page will be located there. Otherwise
        the country will be used as parent.
        """
        base_url = self.survey_url()
        if base_url is not None and \
                aq_inner(self._my_context).absolute_url().startswith(base_url):
            return base_url
        base_url = self.country_url
        if base_url is not None:
            return base_url
        return self.client_url

        if self.anonymous:
            base_url = self.country_url
            if base_url is not None:
                return base_url
            return self.client_url
        return self._base_url()

    @reify
    def base_url(self):
        if self.anonymous:
            base_url = self.country_url
            if base_url is not None:
                return base_url
            return self.client_url
        return self._base_url()

    @reify
    def resources_url(self):
        return "{}/{}".format(
            self.client_url, self.resources_name)

    @reify
    def js_resources_url(self):
        return "{}/{}".format(
            self.client_url, self.js_resources_name)

    @reify
    def is_outside_of_survey(self):
        if self._base_url() != self.survey_url():
            return True
        if (
            self.request.get('ACTUAL_URL').split('/')[-1] ==
            self.survey_url().split('/')[-1]
        ):
            return True
        return False

    @reify
    def get_survey_title(self):
        survey = self._survey
        if not survey:
            return None
        if (
            getattr(self, 'session', None) and
            "/".join(survey.getPhysicalPath()).endswith(self.session.zodb_path)
        ):
            return self.session.title
        return survey.title

    def get_phase(self):
        head, tail = path.split(self.request.PATH_INFO)
        while tail:
            tail = tail.replace('@', '')
            if tail in NAME_TO_PHASE:
                return NAME_TO_PHASE[tail]
            head, tail = path.split(head)
        return ""

    @property
    def came_from_param(self):
        param = ''
        if self.came_from:
            survey_url = self.survey_url()
            if survey_url:
                param = 'came_from={0}'.format(survey_url)
            else:
                param = 'came_from={0}'.format(self.came_from)

        return param

    @reify
    def help_url(self):
        """Return the URL to the current online help page. If we are in a
        survey the help page will be located there. Otherwise the country
        will be used as parent."""
        return '%s/help' % self._base_url()

    @reify
    def about_url(self):
        """Return the URL to the current online about page. If we are in a
        survey the help page will be located there. Otherwise the country
        will be used as parent."""
        return '%s/about' % self._base_url()

    @reify
    def authenticated(self):
        """Check if the current user is authenticated."""
        user = getSecurityManager().getUser()
        return user is not None and user.getUserName() != 'Anonymous User'

    @reify
    def country_url(self):
        """Return the absolute URL for country page."""
        sector = self.sector
        if sector is not None:
            return aq_parent(sector).absolute_url()

        for parent in aq_chain(aq_inner(self._my_context)):
            if IClientCountry.providedBy(parent):
                return parent.absolute_url()

        return None

    @reify
    def session_overview_url(self):
        """Return the absolute URL for the session overview."""
        return self.country_url

    @reify
    def sector_url(self):
        """Return the URL for the current survey."""
        sector = self.sector
        if sector is None:
            return None
        return sector.absolute_url()

    @reify
    def _survey(self):
        survey = getattr(self.request, 'survey', None)
        if survey is not None:
            if ISurvey.providedBy(survey):
                return survey

        if self.session is None:
            return None

        try:
            return self.request.client.restrictedTraverse(
                self.session.zodb_path.split('/'))
        except KeyError as e:
            # This can happen when a survey has been unpublished while the
            # current user still has it in his session.
            logger.error(e)
            return None

    @memoize
    def survey_url(self, phase=None):
        """Return the URL for the curreny survey.

        If a phase is specified the URL for the first page of that
        phase is returned.
        """
        survey = self._survey
        if not survey:
            return None

        url = survey.absolute_url()
        if phase is not None:
            url += '/%s' % phase
        return url

    @memoize
    def survey_zodb_path(self):
        """
            Construct zodb_path from current survey.
            Helper method, since I don't always trust self.session.zodb_path
        """
        elems = []
        obj = self._survey
        while not IClient.providedBy(obj):
            elems.append(obj.id)
            obj = aq_parent(obj)
        elems.reverse()
        return "/".join(elems)

    @reify
    def in_session(self):
        """Check if there is an active survey session."""
        return self._survey is not None

    @reify
    def appendix_documents(self):
        """Return a list of items to be shown in the appendix."""
        documents = api.portal.get().documents

        lt = getToolByName(self._my_context, 'portal_languages')
        lang = lt.getPreferredLanguage()
        if '-' in lang:
            languages = [lang, lang.split('-')[0]]
        else:
            languages = [lang]

        for lang in languages:
            docs = documents.get(lang, None)
            if docs is None:
                continue
            appendix = docs.get('appendix', None)
            if appendix is not None:
                break
        else:
            return []

        base_url = self._base_url()
        return [{'url': '%s/appendix/%s' % (base_url, page.id),
                 'title': page.Title()}
                for page in appendix.values()]

    @reify
    def is_iphone(self):
        """Check if the current request is from an iPhone or similar device
        (such as an iPod touch).
        """
        agent = self.request.get_header('User-Agent', '')
        return 'iPhone' in agent

    def months(self, length='wide'):
        calendar = self.request.locale.dates.calendars['gregorian']
        months = calendar.monthContexts['format'].months[length]
        return sorted(months.items())

    @reify
    def get_sector_logo(self):
        sector = self.sector
        if sector is None:
            return None
        images = getMultiAdapter((sector, self.request), name="images")
        try:
            return images.scale("logo", height=100, direction="up") or None
        except POSKeyError:
            return None

    def messages(self):
        status = IStatusMessage(self.request)
        messages = status.show()
        for m in messages:
            m.id = idnormalizer.normalize(m.message)
        return messages

    def _getLanguages(self):
        lt = getToolByName(self._my_context, "portal_languages")
        lang = lt.getPreferredLanguage()
        if "-" in lang:
            return [lang, lang.split("-")[0], "en"]
        else:
            return [lang, "en"]

    def _findMOTD(self):
        documents = api.portal.get().documents

        motd = None
        for lang in self._getLanguages():
            docs = documents.get(lang, None)
            if docs is None:
                continue
            motd = docs.get("motd", None)
            if motd is not None:
                return motd

    def splash_message(self):
        message = None
        motd = self._findMOTD()
        if motd:
            now = datetime.now()
            message = dict(
                title=StripMarkup(motd.description), text=motd.body,
                id='motd{0}{1}'.format(
                    motd.modification_date.strftime('%Y%m%d%H%M%S'),
                    now.strftime('%Y%m%d'))
            )
        return message

    def tool_notification(self):
        message = None
        obj = self._my_context
        if isinstance(obj, PathGhost):
            obj = self.context.aq_parent
        if ISurvey.providedBy(obj) and obj.hasNotification():
            now = datetime.now()
            message = dict(
                title=obj.tool_notification_title,
                text=obj.tool_notification_message,
                id='tool{}{}{}'.format(
                    obj.modification_date.strftime('%Y%m%d%H%M%S'),
                    now.strftime('%Y%m%d'),
                    ''.join(obj.getPhysicalPath()[2:])))
        return message

    def closetext(self):
        return api.portal.translate(_(u"button_close", default=u"Close"))

    def email_sharing_text(self):
        return api.portal.translate(_(u"I wish to share the following with you"))

    def getSecret(self):
        return getSecret()

    @memoize_contextless
    def get_current_account(self):
        return get_current_account()

    @memoize_contextless
    def get_my_group_id(self):
        account = self.get_current_account()
        return account and account.group_id or ""

    @memoize
    def get_session_group_id(self):
        return getattr(self.session, 'group_id', '')

    @memoize_contextless
    def is_owner(self, session=None):
        ''' Check if the current user is the owner of the session
        '''
        if session is None:
            session = self.session
        if not session:
            return False
        return self.get_current_account() == session.account

    @memoize
    def can_view_session(self, session=None):
        account = self.get_current_account()
        if not account:
            return False
        if session is None:
            session = self.session
        if session is None and self.request.get('sessionid'):
            session = self.session_by_id(self.request.get('sessionid'))
        return (
            session in account.sessions or
            session in account.acquired_sessions
        )

    @memoize
    def can_edit_session(self, session=None):
        return self.can_view_session(session=session)

    @memoize
    def can_publish_session(self, session=None):
        return self.can_edit_session(session=session)

    @memoize
    def can_delete_session(self, session=None, sessionid=''):
        return self.can_edit_session(session=session)

    @memoize
    def can_duplicate_session(self, session=None, sessionid=''):
        return self.use_clone_feature

    def resume(self, session):
        ''' Resume a session for the current user if he is allowed to
        '''
        if not self.can_view_session(session):
            raise ValueError('Can only resume session for current user.')

        self.request.other["euphorie.session"] = session
        setCookie(
            self.request.response,
            self.getSecret(),
            SESSION_COOKIE,
            session.id,
        )

    def as_md(self, text):
        """ Return a text with Carriage Returns formatted as a Markdown.
        """
        return u"\r\n".join([x for x in text.split('\r')])

    def show_logo(self):
        """ In plain Euphorie, the logo is always shown
        """
        return True

    @memoize_contextless
    def date_picker_i18n_json(self):
        """ Taken from:
        https://github.com/ploneintranet/ploneintranet/blob/master/src/ploneintranet/layout/browser/date_picker.py  # noqa: E501

        Use this like:
        <input class="pat-date-picker"
               ...
               data-pat-date-picker="...; i18n: ${portal_url}/@@date-picker-i18n.json; ..."
               />
        """
        json = dumps(
            {
                "previousMonth": api.portal.translate(pae_message("prev_month_link")),
                "nextMonth": api.portal.translate(pae_message("next_month_link")),
                "months": [
                    api.portal.translate(pl_message(month))
                    for month in [
                        "month_jan",
                        "month_feb",
                        "month_mar",
                        "month_apr",
                        "month_may",
                        "month_jun",
                        "month_jul",
                        "month_aug",
                        "month_sep",
                        "month_oct",
                        "month_nov",
                        "month_dec",
                    ]
                ],
                "weekdays": [
                    api.portal.translate(pl_message(weekday))
                    for weekday in [
                        "weekday_sun",
                        "weekday_mon",
                        "weekday_tue",
                        "weekday_wed",
                        "weekday_thu",
                        "weekday_fri",
                        "weekday_sat",
                    ]
                ],
                "weekdaysShort": [
                    api.portal.translate(pl_message(weekday_abbr))
                    for weekday_abbr in [
                        "weekday_sun_abbr",
                        "weekday_mon_abbr",
                        "weekday_tue_abbr",
                        "weekday_wed_abbr",
                        "weekday_thu_abbr",
                        "weekday_fri_abbr",
                        "weekday_sat_abbr",
                    ]
                ],
            }
        )
        return json

    def __call__(self):
        return self


class Appendix(WebHelpers):
    """ Browser View for showing the appendix with various links to
    copyright, license, etc.
    Since this is very client-specific, it gets its own template for easy
    customisation.

    """

    def __call__(self):
        return self


class Logo(WebHelpers):
    """ Browser View for showing the markup for the logo
    Since this is very client-specific, it gets its own template for easy
    customisation.

    """

    def __call__(self):
        return self


class UserMenu(WebHelpers):
    """
    View class for the User Menu

    """

    def __call__(self):
        return self.index()


class HelpMenu(WebHelpers):
    """
    View class for the User Menu

    """

    def __call__(self):
        return self.index()



# coding=utf-8
from Acquisition import aq_inner
from datetime import datetime
from euphorie import MessageFactory as _
from euphorie.client import utils
from euphorie.client.browser.country import SessionsView
from euphorie.client.model import get_current_account
from plone import api
from plone.autoform.form import AutoExtensibleForm
from plone.memoize.view import memoize
from plone.memoize.view import memoize_contextless
from plone.supermodel import model
from Products.Five import BrowserView
from z3c.form.form import EditForm
from zope import schema
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent


class IStartFormSchema(model.Schema):
    title = schema.TextLine(
        title=_(
            "label_session_title",
            default=u"Enter a title for your Risk Assessment"),
        required=True,
    )


class SurveySessionsView(SessionsView):
    """ Template corresponds to proto:_layout/tool.html
    """

    variation_class = ""

    @memoize
    def get_sessions(self):
        """ Filter user's sessions to match only those from the current survey
        """
        sessions = super(SurveySessionsView, self).get_sessions()
        survey = aq_inner(self.context)
        my_path = utils.RelativePath(self.request.client, survey)
        my_sessions = sorted(
            [x for x in sessions if x.zodb_path == my_path],
            key=lambda s: s.modified,
            reverse=True)
        return my_sessions


class Start(AutoExtensibleForm, EditForm):
    """Survey start screen.

    This view shows basic introduction text and any extra information provided
    the sector if present. After viewing this page the user is forwarded to the
    profile page.

    View name: @@start
    """
    ignoreContext = True
    schema = IStartFormSchema
    variation_class = "variation-risk-assessment"

    @property
    def template(self):
        return self.index

    @property
    @memoize
    def webhelpers(self):
        return api.content.get_view(
            'webhelpers',
            self.context,
            self.request,
        )

    @property
    @memoize
    def has_profile(self):
        return len(self.context.ProfileQuestions())

    @memoize
    def has_introduction(self):
        survey = aq_inner(self.context)
        return utils.HasText(getattr(survey, "introduction", None))

    def update(self):
        super(Start, self).update()
        self.message_required = api.portal.translate(
            _(
                u"message_field_required",
                default=u"Please fill out this field."
            ),
        )
        if self.request.environ["REQUEST_METHOD"] != "POST":
            return
        data, errors = self.extractData()
        if errors:
            return
        session = self.webhelpers.session
        changed = False
        for key in data:
            value = data[key]
            if getattr(session, key, None) != value:
                changed = True
                setattr(session, key, value)

        if changed:
            api.portal.show_message(
                _("Session data successfully updated"),
                request=self.request,
                type='success',
            )
        # Optimize: if the form was auto-submitted, we know that we want to
        # show the "start" page again
        if "form.button.submit" not in self.request:
            self.request.response.redirect(
                "%s/@@start" % self.context.absolute_url()
            )
        else:
            self.request.response.redirect(
                "%s/@@profile" % self.context.absolute_url()
            )


class PublicationMenu(BrowserView):

    @property
    @memoize
    def webhelpers(self):
        return api.content.get_view(
            'webhelpers',
            self.context,
            self.request,
        )

    @property
    @memoize_contextless
    def portal(self):
        ''' The currenttly authenticated account
        '''
        return api.portal.get()

    def notify_modified(self, session):
        notify(ObjectModifiedEvent(session))

    def redirect(self):
        target = (
            self.request.get('HTTP_REFERER') or self.context.absolute_url()
        )
        return self.request.response.redirect(target)

    def reset_date(self, sessionid):
        ''' Reset the session date to now
        '''
        session = self.session(sessionid)
        session.published = datetime.now()
        session.last_publisher = get_current_account()
        self.notify_modified(session)
        return self.redirect()

    def set_date(self, sessionid):
        ''' Set the session date to now
        '''
        return self.reset_date(sessionid)

    def unset_date(self, sessionid):
        ''' Unset the session date
        '''
        session = self.session(sessionid)
        session.published = None
        session.last_publisher = None
        self.notify_modified(session)
        return self.redirect()

    @memoize
    def session(self, sessionid):
        return self.webhelpers.session_by_id(sessionid)
        # return SessionManager.session



# coding=utf-8
from ...country import Country
from ...tests.utils import createSector
from ..authentication import generate_token
from ..country import country_info
from ..country import View
from euphorie.testing import EuphorieFunctionalTestCase
from zope.publisher.browser import TestRequest

import json
import mock
import unittest


class country_info_tests(unittest.TestCase):

    def country_info(self, *a, **kw):
        return country_info(*a, **kw)

    def test_data(self):
        country = Country(
            id='nl', title=u'The Netherlands', country_type='eu-member'
        )
        info = self.country_info(country)
        self.assertEqual(set(info), set(['id', 'title', 'country-type']))
        self.assertEqual(info['id'], 'nl')
        self.assertEqual(info['title'], u'The Netherlands')
        self.assertEqual(info['country-type'], 'eu-member')


class ViewTests(unittest.TestCase):

    def test_do_GET_basic(self):
        view = View('context', TestRequest())
        with mock.patch(
            'euphorie.content.api.country.country_info',
            return_value={
                'foo': 'bar'
            }
        ) as mock_country_info:
            self.assertEqual(view.do_GET(), {'type': 'country', 'foo': 'bar'})
            mock_country_info.assert_called_once_with('context')

    def test_do_GET_details(self):
        view = View('context', TestRequest())
        view.request.form['details'] = ''
        with mock.patch(
            'euphorie.content.api.country.country_info', return_value={}
        ):
            with mock.patch(
                'euphorie.content.api.country.list_managers',
                return_value='mgr-list'
            ):
                with mock.patch(
                    'euphorie.content.api.country.list_sectors',
                    return_value='sector-list'
                ):
                    info = view.do_GET()
                    self.assertEqual(info['managers'], 'mgr-list')
                    self.assertEqual(info['sectors'], 'sector-list')


class ViewBrowserTests(EuphorieFunctionalTestCase):

    def test_require_authentication(self):
        browser = self.get_browser()
        browser.raiseHttpErrors = False
        browser.open('http://nohost/plone/api/countries/nl')
        self.assertTrue(browser.headers['Status'].startswith('401'))

    def test_authenticated_user(self):
        sector = createSector(self.portal, login='sector', password=u'sector')
        browser = self.get_browser()
        browser.handleErrors = False
        browser.addHeader('X-Euphorie-Token', generate_token(sector))
        browser.open('http://nohost/plone/api/countries/nl')
        response = json.loads(browser.contents)
        self.assertEqual(
            set(response), set(['type', 'id', 'title', 'country-type'])
        )
        self.assertEqual(response['type'], u'country')
        self.assertEqual(response['id'], u'nl')



from setuptools import find_packages
from setuptools import setup


version = '11.0.5'

setup(
    name="Euphorie",
    version=version,
    description="Euphorie Risk Assessment tool",
    long_description=open("README.rst").read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 5.1",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python :: 2 :: Only",
        "Programming Language :: Python :: 2.7",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="euphorie OiRA Interactive Risk Assessment",
    author="Wichert Akkerman and Syslab.com",
    author_email="info@syslab.com",
    url="http://euphorie.readthedocs.org/en/latest/",
    license="GPL",
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=[],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "Plone >=5.1.0",
        "Zope2 >=2.13.19",
        'AccessControl >= 3.0',
        'Products.membrane >=2.1.3',
        "SQLAlchemy >=0.5.2",
        "anytree",
        "decorator",
        "py-bcrypt",
        "five.grok",
        "htmllaundry [z3cform] >=1.1dev-r11183",
        "lxml",
        "plone.app.dexterity [grok, relations]",
        "plone.app.folder",
        "plone.app.imaging",
        "plone.app.imagecropping",
        "plone.app.redirector >= 1.0.12dev-r27477",
        "plone.app.vocabularies",
        "plone.app.z3cform",
        "plone.behavior >=1.0b4",
        "plone.dexterity >=1.0b6",
        "plone.directives.dexterity",
        "plone.directives.form",
        "plone.formwidget.namedfile",
        "plone.memoize",
        "plone.namedfile[blobs]",
        "plone.api",
        "python-docx",
        "repoze.formapi >=0.4.2",
        "setuptools",
        "simplejson",
        "z3c.form >= 2.1.0",
        "z3c.schema",
        "z3c.saconfig",
        "zope.configuration >= 3.6",
        "zope.i18nmessageid",
        "zope.interface",
        "zope.schema",
        "NuPlone >=1.6.0",
        "plone.uuid",
        "pyrtf-ng",
        "z3c.appconfig >=1.0",
        "openpyxl",
        "Chameleon >=2.12",
    ],
    tests_require=[
        "collective.testcaselayer >=1.6",
        "Products.PloneTestCase >=0.9.9",
        'mock',
    ],
    extras_require={
        "sphinx": [
            "Sphinx >=1.0",
            "repoze.sphinx.autointerface",
        ],
        "tests": [
            "plone.app.testing",
            "collective.testcaselayer",
            "Products.PloneTestCase >=0.9.9",
            "mock",
        ],
    },
    entry_points="""
      [z3c.autoinclude]
      target = plone

      [zopectl.command]
      initdb = euphorie.deployment.commands.initdb:main
      xmlimport = euphorie.deployment.commands.xmlimport:main

      [console_scripts]
      upgradedb = euphorie.deployment.commands.upgradedb:main
      """,
)
